@@ -1,17 +1,15 @@
 from __future__ import annotations
 
 import copy
-import sys
 from typing import Any
 
-import pytest
 import torch
 from torch import nn
 from torchvision.models import resnet18  # type: ignore[import]
 
 from rebasin import PermutationCoordinateDescent
 from rebasin.weight_matching.permutation_coordinate_descent import calculate_progress
-from tests.fixtures.models import MLP
+from tests.fixtures.models import MLP, ModuleWithWeirdWeightAndBiasNames
 
 
 def test_calculate_progress() -> None:
@@ -33,23 +31,21 @@ def test_calculate_progress() -> None:
 
 
 class TestPermutationCoordinateDescent:
-    seed1 = 1234567
-    seed2 = 9948733
-
-    @pytest.mark.skipif("--full-suite" not in sys.argv, reason="Slow test")
     def test_resnet18(self) -> None:
-        torch.manual_seed(self.seed1)
+        # In a run of 500 tests, this test succeeded every time.
         model_a = resnet18()
-        torch.manual_seed(self.seed2)
         model_b = resnet18()
         self.common_tests(model_a, model_b, torch.randn(1, 3, 224, 224))
 
     def test_mlp(self) -> None:
-        torch.manual_seed(self.seed1)
-        model_a = MLP(5)
-        torch.manual_seed(self.seed2)
-        model_b = MLP(5)
-        self.common_tests(model_a, model_b, torch.randn(5))
+        # Wide layers, or else some permutations will be the identity.
+        # If every permutation is the identity, the test will fail.
+        # Increasing layer size is better than increasing layer count.
+        # At weights of size (25, 25), the test succeeds 500 out of 500 attempts.
+        # For (5, 5) weights, the test fails fairly often.
+        model_a = MLP(25, num_layers=5)
+        model_b = MLP(25, num_layers=5)
+        self.common_tests(model_a, model_b, torch.randn(25))
 
     def test_multihead_attention(self) -> None:
         embed_dim = num_heads = 8
@@ -63,6 +59,12 @@ class TestPermutationCoordinateDescent:
 
         self.common_tests(model_a, model_b, (query, key, value))
 
+    def test_module_with_weird_weight_and_bias_names(self) -> None:
+        model_a = ModuleWithWeirdWeightAndBiasNames()
+        model_b = ModuleWithWeirdWeightAndBiasNames()
+
+        self.common_tests(model_a, model_b, torch.randn(15))
+
     def common_tests(
             self,
             model_a: nn.Module,
@@ -86,21 +88,18 @@ class TestPermutationCoordinateDescent:
         # - model_b is closer to model_a than it was before the optimization
         pcd.apply_permutations()
         assert (
-                self.model_distance(model_a, model_b)
-                < self.model_distance(model_a, model_b_old)
+                self.model_similarity(model_a, model_b)
+                > self.model_similarity(model_a, model_b_old)
         )
 
     @staticmethod
-    def model_distance(model_a: nn.Module, model_b: nn.Module) -> float:
+    def model_similarity(model_a: nn.Module, model_b: nn.Module) -> float:
         """Calculate the distance between two models."""
-        # TODO: this measure of distance might not be ideal.
-        #   It might be better to calculate the cost matrix between all parameters,
-        #   and then calculate the distance between the cost matrices and the identity.
-        distance = 0.0
+        total_dist = 0.0
+
         for parameter_a, parameter_b in zip(
                 model_a.parameters(), model_b.parameters(), strict=True
         ):
-            distance += torch.norm(  # type: ignore[no-untyped-call]
-                parameter_a - parameter_b
-            ).item()
-        return distance
+            p_a, p_b = parameter_a.reshape(-1), parameter_b.reshape(-1)
+            total_dist += float(p_a @ p_b)
+        return abs(total_dist)
